/*******************************************************************************
Description:

  Integer function assign_geos_vals.c
  Assigns GEOS variable values to each pixel location in the input L1b granule.
  Also spatially interpolates GEOS values to pixel locations if desired.
  Called from get_GEOS.c

Input arguments:
  none

Output arguments:
  none

Function output:
  int return_code  successful completion is zero, otherwise non-zero

  Pointers to inputs are stored in ancillary.h.
  Pointers to output variables are stored in ancillary.h.

Revision History:
  10/2012 R. Frey  Original version
  01/2021 R. Frey  Made changes to accomodate sector processing.
                   Code cleanup.
  06/2021 R. Frey  Modified from assign_gdas_vals.c
  06/2021 R. Frey  Added 'snowfr' and 'icefr'.

Calls:
  none

*******************************************************************************/

// Includes
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

//MVCCM includes
#include "ancillary.h"
//#include "granule_geolocation.h"
//#include "granule.h"
#include "mask_processing_constants.h"

/******************************************************************************/

int assign_geos_vals(float *lat, float *lon, int res)

{

/******************************************************************************/

//  Declarations and initializations.

    extern double fabs(double);

    const int eles=3200*res;
    const int lines=3232*res;

    int row, col;
    int i, j, ij, i1, i2, j1, j2, ic, jc;
    int j1c, j2c, i1c, i2c;
    int line1, line2, ele1, ele2;
    long int n_pixels = eles * lines;
    long int nwp_index, nwp_index_c;
    long int nwp_index_c1, nwp_index_c2, nwp_index_c3, nwp_index_c4;
    long int indx00, indx01, indx10, indx11;
    long int idx, idxloc;
    float lst_sum;
    double x0 = -180.0;
    double dx = 0.625;
    double dxc = 0.3125;
    double y0 = -90.0;
    double dy = 0.50;
    double dyc = 0.25;
    double x, lat1, lat2, lat00, lat01, lat10, lat11, dyi, pyi, py;
    double y, lon1, lon2, lon00, lon01, lon10, lon11, dxi, pxi, px;
    double t, tx, ty, t00, t01, t10, t11;

    int return_code = 0;

/******************************************************************************/

//  Allocate memory for assigned data.  Structure variables are stored in
//  ancillary.h.
    if ((grn_anc.sfct = (float *) malloc(sizeof(float) * n_pixels)) == NULL) {
      printf("Could not allocate memory for sfct\n");
      return(-1);
    }
    if ((grn_anc.tpw = (float *) malloc(sizeof(float) * n_pixels)) == NULL) {
      printf("Could not allocate memory for tpw\n");
      return(-2);
    }
    if ((grn_anc.snowfr = (float *) malloc(sizeof(float) * n_pixels)) == NULL) {
      printf("Could not allocate memory for snowfr\n");
      return(-2);
    }
    if ((grn_anc.icefr = (float *) malloc(sizeof(float) * n_pixels)) == NULL) {
      printf("Could not allocate memory for icefr\n");
      return(-2);
    }
    if ((grn_anc.geos_ocnfr = (float *) malloc(sizeof(float) * n_pixels)) == NULL) {
      printf("Could not allocate memory for geos_ocnfr\n");
      return(-2);
    }
    if ((grn_anc.landicefr = (float *) malloc(sizeof(float) * n_pixels)) == NULL) {
      printf("Could not allocate memory for landicefr\n");
      return(-2);
    }

/******************************************************************************/

//   Get TPW, snow fraction, ice fraction, and bilinearly interpolated surface
//   temperature.

/******************************************************************************/

// Use 0-based line and element numbers.
   idx = 0;
   idxloc = 0;
   ij = 0;

   for ( row=0; row<lines; ++row) {
     for ( col=0; col<eles; ++col) {

/******************************************************************************/

//       Find GEOS 0.5-degree grid box corresponding to current location.
         y = min(max(lat[idx], -90.00), 90.00);
         j = (int) ((y - y0 + dy) / dy);

         x = min(max(lon[idx], -180.00), 179.375);
         i = (int) ((x - x0 + dx) / dx);

         nwp_index = (j-1)*nwp_cols + (i-1);
         if(nwp_index < 0 || nwp_index > nwp_cells) {
           printf("NWP grid box out of range: %ld\n", nwp_index);
           return (-3);
         }

//       Find GEOS 0.25-degree grid box corresponding to current location.
         jc = (int) ((y - y0 + dyc) / dyc);
         ic = (int) ((x - x0 + dxc) / dxc);
         nwp_index_c = ((jc-1)*nwp_cols*2) + (ic-1);

/******************************************************************************/
//       Don't spatially interpolate the following variables.
         grn_anc.tpw[idxloc] = *(ti_tpw + nwp_index);
         grn_anc.snowfr[idxloc] = *(g_snow + nwp_index);
         grn_anc.icefr[idxloc] = *(g_ice + nwp_index);
         grn_anc.geos_ocnfr[idxloc] = (*(g_frocn + nwp_index_c));
         grn_anc.landicefr[idxloc] = (*(g_frlandice + nwp_index_c));

//       Get non-interpolated value in case interpolation is not performed.
         grn_anc.sfct[idxloc] = *(ti_sfct + nwp_index);

/******************************************************************************/

//       Find four GEOS values that surround the pixel of interest.  Begin by
//       finding the two surrounding latitude bands and computing the fractional
//       distance between the pixel and the closest latitude band.
         lat1 = y0 + ((j-1) * dy);
         lat2 = y0 + ((j+1) * dy);
         if( fabs(y-lat1) <= fabs(y-lat2) ) {
           j1 = j-1;
           j2 = j;
         }else {
           j1 = j;
           j2 = j+1;
         }
         lat00 = y0 + (j1 * dy);
         lat10 = y0 + (j2 * dy);
         dyi = lat00 - lat10;
         pyi = lat00 - y;
         if(pyi > 0.0) {
           py = pyi / dyi;
         }
         else {
           py = dy;
         }
         if(j1 < 0 || fabs(dyi) < 0.000001 || py > 1.00 || py < 0.00) {
           printf("problem with j index in spatial interpolation\n");
           printf("%d %d %ld %d %d %d %f %f %f %f %f %f\n", row,col,idx,j,j1,j2,lat00,lat10,dyi,y,pyi,py);
         }

//       Find surrounding longitudes and compute the fractional distance between
//       the pixel of interest and the closest longitude box.
         lon1 = x0 + ((i-1) * dx);
         lon2 = x0 + ((i+1) * dx);
//       if(lon1 < 0.0) lon1 = lon1 + 360.0;
         if( fabs(x-lon1) < fabs(x-lon2) ) {
           i1 = i-1;
           i2 = i;
         }else {
           i1 = i;
           i2 = i+1;
         }

         lon00 = x0 + (i1 * dx);
         lon10 = x0 + (i2 * dx);
         dxi = lon00 - lon10;
         pxi = lon00 - x;
         if(pxi > 0.0) {
           px = pxi/dxi;
         }
         else {
           px = dx;
         }
         if(fabs(dxi) < 0.000001 || px > 1.00 || px < 0.00) {
           printf("problem with i index in spatial interplation\n");
           printf("%d %d %ld %d %d %d %f %f %f %f %f %f\n", row,col,idx,i,i1,i2,lon00,lon10,dxi,x,pxi,px);
         }

//       Following are indices of surrounding NWP gridboxes.
         indx00 = (j1 * nwp_cols) + i1;
         indx01 = (j1 * nwp_cols) + i2;
         indx10 = (j2 * nwp_cols) + i1;
         indx11 = (j2 * nwp_cols) + i2;

//       Interpolate only if region contains only land or water.
//       Get ocean fraction for the four regions.
         j1c = (j1*2)-1;
         j2c = j2*2;
         i1c = (i1*2)-1;
         i2c = i2*2;
         if(j1c < 0) j1c = 1;
         if(i1c < 0) i1c = 1;
         nwp_index_c1 = (j1c-1)*nwp_cols*2 + (i1c-1);
         nwp_index_c2 = (j1c-1)*nwp_cols*2 + (i2c-1);
         nwp_index_c3 = (j2c-1)*nwp_cols*2 + (i1c-1);
         nwp_index_c4 = (j2c-1)*nwp_cols*2 + (i2c-1);
         lst_sum =  *(g_frocn+nwp_index_c1) + *(g_frocn+nwp_index_c2) + *(g_frocn+nwp_index_c3) + *(g_frocn+nwp_index_c4);
//       printf("\nlat,lon: %f %f %f %f %f %f\n", grn_geo.lat[idx], grn_geo.lon[idx],lat00,lat10,lon00,lon10);
//       printf("nfrocean1: %d %d %d %d %d %d %d %d\n", j1,j2,i1,i2,j1c,j2c,i1c,i2c);
//       printf("frocean2: %f %f %f %f %f\n", *(g_frocn+nwp_index_c1),*(g_frocn+nwp_index_c2),*(g_frocn+nwp_index_c3),
//             *(g_frocn+nwp_index_c4),lst_sum);

         if(lst_sum == 4.0 || lst_sum == 0.0) {

           t00 = ti_sfct[indx00];
           t01 = ti_sfct[indx01];
           tx = ((1.0 - px) * t00) + (px * t01);

           t10 = ti_sfct[indx10];
           t11 = ti_sfct[indx11];
           ty = ((1.0 - px) * t10) + (px * t11);
           t = ((1.0 - py) * tx) + (py * ty);

           grn_anc.sfct[idxloc] = t;

//         if(ij < 10)
//           printf("interp: %f %f %f %f %f %f %f %f\n", lst_sum, t00, t01, tx, t10, t11, ty, t);
//         ij++;

         }

         idx++;
         idxloc++;

     }
   }

/******************************************************************************/

    return (return_code);

/******************************************************************************/

}
