/******************************************************************************

Description:

  Integer function read_GEOS_constants.c
  Reads desired variables from time-invariant (constant) file (calling argument).

Input arguments:
  geos_file_cnst        input GEOS constants file name

Output arguments:
  none

Function output:
  int return_code       successful completion is zero, otherwise non-zero

  Output variables in ancillary.h

Revision History:
  07/2021     R. Frey   Original version

Calls:
  none

*******************************************************************************/

// Includes

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stddef.h>
#include <math.h>
#include <float.h>

//MVCCM includes
//#include "granule_geolocation.h"
//#include "granule.h"
#include "ancillary.h"

//netcdf includes
#include <netcdf.h>

/******************************************************************************/

int read_GEOS_constants(char *geos_file_cnst)

{

/******************************************************************************/

//  Declarations and initializations.

/******************************************************************************/

    size_t lat_len, lon_len, start[3], edge[3];
    int status, idf, latln_id, lonln_id, nxc, nyc, nxnyc, varid;
    int return_code = 0;

/******************************************************************************/

    start[0] = 0;
    start[1] = 0;
    start[2] = 0;
    edge[0] = 0;
    edge[1] = 0;
    edge[2] = 0;

/******************************************************************************/

//  Open file.
//  printf("Opening GEOS constants file: %s\n", geos_file_cnst);
    status = nc_open(geos_file_cnst, NC_NOWRITE, &idf);
    if (status != NC_NOERR) {
      printf("Could not open GEOS constants file: %s %d %d\n", geos_file_cnst,
             idf, status);
      return (-1);
    }

/******************************************************************************/

//  Get data array dimensions.  All meteorologial variables are the same size.
//  First dimension (time) is 1.

    status = nc_inq_dimid(idf, "lat", &latln_id);
    if(status < 0) {
      printf("Could not acquire lat-dimension data: %s\n", geos_file_cnst);
      return_code = -2;
      return (return_code);
    }
    status = nc_inq_dimlen(idf, latln_id, &lat_len);

    status = nc_inq_dimid(idf, "lon", &lonln_id);
    if(status < 0) {
      printf("Could not acquire lon-dimension data: %s\n", geos_file_cnst);
      return_code = -3;
      return (return_code);
    }
    status = nc_inq_dimlen(idf, lonln_id, &lon_len);

/******************************************************************************/

    nxc = lon_len;
    nyc = lat_len;
    nxnyc = lat_len * lon_len;

    if ((g_frocn = (float *) malloc(sizeof(float) * nxnyc)) == NULL) {
      printf("Could not allocate memory for g_frocn\n");
      return(-4);
    }
    if ((g_frlandice = (float *) malloc(sizeof(float) * nxnyc)) == NULL) {
      printf("Could not allocate memory for g_frlandice\n");
      return(-4);
    }
    if ((g_phis = (float *) malloc(sizeof(float) * nxnyc)) == NULL) {
      printf("Could not allocate memory for g_glst\n");
      return(-5);
    }

/******************************************************************************/

    edge[0] = 1;
    edge[1] = lat_len;
    edge[2] = lon_len;

/******************************************************************************/

//  Read ocean fractions.

    status = nc_inq_varid (idf, "FROCEAN", &varid);
    if(status != NC_NOERR) {
      printf("Could not find FROCEAN variable in %s\n", geos_file_cnst);
      return_code = -8;
      return (return_code);
    }

    status = nc_get_vara_float(idf, varid, start, edge, g_frocn);
    if (status != NC_NOERR) {
      printf("could not get FROCEAN data in %s\n", geos_file_cnst);
      printf("status: %d\n", status);
      return_code = -9;
      return (return_code);
    }

/******************************************************************************/

//  Read permanent land ice fractions.

    status = nc_inq_varid (idf, "FRLANDICE", &varid);
    if(status != NC_NOERR) {
      printf("Could not find FRLANDICE variable in %s\n", geos_file_cnst);
      return_code = -8;
      return (return_code);
    }

    status = nc_get_vara_float(idf, varid, start, edge, g_frlandice);
    if (status != NC_NOERR) {
      printf("could not get FROCEAN data in %s\n", geos_file_cnst);
      printf("status: %d\n", status);
      return_code = -9;
      return (return_code);
    }

/******************************************************************************/

//  Read surface geopotential heights.

    status = nc_inq_varid (idf, "PHIS", &varid);
    if(status != NC_NOERR) {
      printf("Could not find PHIS variable in %s\n", geos_file_cnst);
      return_code = -10;
      return (return_code);
    }

    status = nc_get_vara_float(idf, varid, start, edge, g_phis);
    if (status != NC_NOERR) {
      printf("could not get PHIS data in %s\n", geos_file_cnst);
      printf("status: %d\n", status);
      return_code = -11;
      return (return_code);
    }

/******************************************************************************/

//  printf("GEOS frc ocn data: %f %f %f\n", g_frocn[0], g_frocn[9], g_frocn[nxnyc-1]);
//  printf("GEOS PHIS data: %f %f %f\n", g_phis[0], g_phis[9], g_phis[nxnyc-1]);
//  fflush;

/******************************************************************************/

//  Close files.
    status = nc_close(idf);

/******************************************************************************/

    return (return_code);

/******************************************************************************/

}
