/******************************************************************************

Description:

  Integer function read_GEOS_lndocn.c
  Reads desired variables from input files (in calling argument list).

Input arguments:
  geos_file_lnd         input GEOS land property file name
  geos_file_ocn         input GEOS ocean property file name

Output arguments:
  none

Function output:
  int return_code       successful completion is zero, otherwise non-zero

  Output variables in ancillary.h

Revision History:
  06/2021     R. Frey   Original version

Calls:
  none

*******************************************************************************/

// Includes

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stddef.h>
#include <math.h>
#include <float.h>

//MVCCM includes
//#include "granule_geolocation.h"
//#include "granule.h"
#include "ancillary.h"

//netcdf includes
#include <netcdf.h>

/******************************************************************************/

int read_GEOS_lndocn(char *geos_file_lnd, char *geos_file_ocn)

{

/******************************************************************************/

//  Declarations and initializations.

/******************************************************************************/

    size_t lat_len, lon_len, start[3], edge[3];
    int status, idfl, idfo, latln_id, lonln_id, nx, ny, nxny, varid;
    int return_code = 0;

/******************************************************************************/

    start[0] = 0;
    start[1] = 0;
    start[2] = 0;
    edge[0] = 0;
    edge[1] = 0;
    edge[2] = 0;

/******************************************************************************/

//  Open files.
//  printf("Opening GEOS land file: %s\n", geos_file_lnd);
    status = nc_open(geos_file_lnd, NC_NOWRITE, &idfl);
    if (status != NC_NOERR) {
      printf("Could not open GEOS land property file: %s %d %d\n", geos_file_lnd,
             idfl, status);
      return (-1);
    }
//  printf("Opening GEOS ocean file: %s\n", geos_file_ocn);
    status = nc_open(geos_file_ocn, NC_NOWRITE, &idfo);
    if (status != NC_NOERR) {
      printf("Could not open GEOS ocean property file: %s %d %d\n", geos_file_ocn,
             idfo, status);
      return (-1);
    }

/******************************************************************************/

//  Get data array dimensions.  All meteorologial variables are the same size.
//  First dimension (time) is 1.

    status = nc_inq_dimid(idfl, "lat", &latln_id);
    if(status < 0) {
      printf("Could not acquire lat-dimension data: %s\n", geos_file_lnd);
      return_code = -2;
      return (return_code);
    }
    status = nc_inq_dimlen(idfl, latln_id, &lat_len);

    status = nc_inq_dimid(idfl, "lon", &lonln_id);
    if(status < 0) {
      printf("Could not acquire lon-dimension data: %s\n", geos_file_lnd);
      return_code = -3;
      return (return_code);
    }
    status = nc_inq_dimlen(idfl, lonln_id, &lon_len);

/******************************************************************************/

    nx = lon_len;
    ny = lat_len;
    nxny = lat_len * lon_len;
    nwp_cells = nxny;
    nwp_cols = nx;
    nwp_rows = ny;

//  printf("dims: %d %d %d %d %d %d\n", idfl, latln_id, lonln_id, lat_len, lon_len, nxny);

    if ((g_snow = (float *) malloc(sizeof(float) * nxny)) == NULL) {
      printf("Could not allocate memory for g_snow\n");
      return(-4);
    }
    if ((g_ice = (float *) malloc(sizeof(float) * nxny)) == NULL) {
      printf("Could not allocate memory for g_ice\n");
      return(-5);
    }

/******************************************************************************/

    edge[0] = 1;
    edge[1] = lat_len;
    edge[2] = lon_len;

//  Read snow fractions.

    status = nc_inq_varid (idfl, "FRSNO", &varid);
    if(status != NC_NOERR) {
      printf("Could not find FRSNO variable in %s\n", geos_file_lnd);
      return_code = -8;
      return (return_code);
    }

    status = nc_get_vara_float(idfl, varid, start, edge, g_snow);
    if (status != NC_NOERR) {
      printf("could not get FRSNO data in %s\n", geos_file_lnd);
      printf("status: %d\n", status);
      return_code = -9;
      return (return_code);
    }

//  Read ice fractions.

    status = nc_inq_varid (idfo, "FRSEAICE", &varid);
    if(status != NC_NOERR) {
      printf("Could not find FRSEAICE variable in %s\n", geos_file_ocn);
      return_code = -10;
      return (return_code);
    }

    status = nc_get_vara_float(idfo, varid, start, edge, g_ice);
    if (status != NC_NOERR) {
      printf("could not get FRSEAICE data in %s\n", geos_file_ocn);
      printf("status: %d\n", status);
      return_code = -11;
      return (return_code);
    }

/******************************************************************************/

//  printf("GEOS snow data: %f %f %f\n", g_snow[0], g_snow[9], g_snow[nxny-1]);
//  printf("GEOS ice data: %f %f %f\n", g_ice[0], g_ice[9], g_ice[nxny-1]);
//  fflush;

/******************************************************************************/

//  Close files.
    status = nc_close(idfl);
    status = nc_close(idfo);

/******************************************************************************/

    return (return_code);

/******************************************************************************/

}
