/**********************************************************************
Description:

  Function snow_mask

  Subroutine which implements a quick and dirty version of the snow
  algorithm by Dorothy Hall, George Riggs and Vince Salmonson.
  Called from get_pxldat.c.

  Version 2 adds snow from NDSI=0.4 to NDSI=0.1 but also as a function
  of NDVI (NDVI=0.1 to NDVI=1.0).

Input arguments:
  satname           Instrument/platform
                      MODIS_Aqua
                      MODIS_Terra
                      VIIRS_NPP
                      VIIRS_N20

  Input values stored in variables found in pixel.h, granule.h,
  mask_processing_constants.h, and thresholds.h.

Output arguments:
  none

  Output stored in 'pxin.ndsi_snow' found in pixel.h.

Revision History:
  10/2007  R. Frey  Converted to C
  10/2012  R. Frey  Adapted for MVCCM software.
  11/2012  R. Frey  Adapted for MVCCM algorithm.
  04/2014  R. Frey  Added Klein et al. NDSI extension as a function of
                    NDVI
  10/2014  R. Frey  Added "mountain" snow test from Thompson et al., 2014
  01/2015  R. Frey  Added surface temperature test to snow detection and
                    added 'lsf' to calling argument list
  10/2015  R. Frey  Added 'Grnlnd_ndsi', 'sm85_11hel', 'sm_lsfcdif', 'sm_wsfcdif',
                          'sm_bt1
  11/2015  R. Frey  Code clean-up.
  04/2018  R. Frey  Added 'prd_ndvi_const'.
  02/2020  R. Frey  Added logic to include NOAA-20.

***********************************************************************/

/* Includes */

#include <stdio.h>
#include <math.h>
//#include "granule.h"
//#include "pixel.h"
//#include "mask_processing_constants.h"
//#include "thresholds.h"

void snow_mask(char *satname, unsigned char lsf)

{

// Declarations

   extern float rintf(float);
   extern int strcmp(char *, char *);

   int I_bad;
   int hi_elev_snow;

   float m04;
   float m01;
   float m02;
   float m20;
   float m29;
   float m31;
   float m35;
   float nirband;
   float ndsi;
   float ndvi;
   float pred_ndvi;
   float diff;
   float diff2;
   float sth85_11;
   float sth37_11;
   float sfcdif;
   float bad_data = -999.0;

// Initializations
   m04 = pxin.rad[pxin.bnd555];
   m01 = pxin.rad[pxin.bnd065];
   m02 = pxin.rad[pxin.bnd086];
   m20 = pxin.rad[pxin.bnd375];
   m29 = pxin.rad[pxin.bnd855];
   m31 = pxin.rad[pxin.bnd11];

   diff = 0.0;
   diff2 = 0.0;
   hi_elev_snow = 0;

// Select NIR channel for NDSI based on platform name.
// Band 6 for Terra, band 7 for Aqua.
//   if(strcmp(satname, "MODIS_Terra") == 0) {
//     nirband = pxin.rad[pxin.bnd164];
//   }
//   else if(strcmp(satname, "MODIS_Aqua") == 0) {
//     nirband = pxin.rad[pxin.bnd213];
//   }
   if(strcmp(satname, "VIIRS_NPP") == 0) {
     nirband = pxin.rad[pxin.bnd164];
   }
   else if(strcmp(satname, "VIIRS_N20") == 0) {
     nirband = pxin.rad[pxin.bnd164];
   }
   else {
     printf("Platform name not recognized in snow_mask \n");
   }

   I_bad = rintf(bad_data);

// Begin snow/ice detection.

// First, check the 11 micron brightness temperature.
   if (rintf(m31) != I_bad && m31 <= sm_bt11[0]) {

//   Get NDSI index
     if (rintf(m04) != I_bad && rintf(nirband) != I_bad) {

       ndsi  = (m04 - nirband) / (m04 + nirband);
       ndvi = (m02 - m01) / (m02 + m01);
       pred_ndvi = prd_ndvi_const[0] + (3.0 * ndsi);

       if(pxin.land && (!pxin.Greenland) && (!pxin.Antarctic) ) {
         if(pxin.elev > 1000.0) {
           hi_elev_snow = 1;
           if(ndsi > 0.0) pxin.ndsi_snow = 1;
         }
         else if ( (ndsi > sm_ndsi[0] && m02 > sm_ref2[0]) ||
              (ndsi > 0.1 && ndsi <= sm_ndsi[0] && ndvi <= pred_ndvi) ) {
           pxin.ndsi_snow = 1;
         }
       }
       else if( pxin.land && pxin.Greenland) {
         if( ndsi > Grnlnd_ndsi[0] ) pxin.ndsi_snow = 1;
       }
       else {
         if (ndsi > sm_ndsi[0] && m02 > sm_ref2[0]) pxin.ndsi_snow = 1;
       }

//     printf("ndsi: %f %f %f %f %f %f %f %f %d\n", m01, m02, m04, nirband, ndsi, ndvi, prd_ndvi_const[0], pred_ndvi, pxin.ndsi_snow);

       if(pxin.ndsi_snow) {

//       If in sunglint region, disregard if between -60 and 50 degrees latitude.
         if(pxin.water && pxin.sunglint && pxin.lat <= 50.0 && pxin.lat >= -60.0) {
           pxin.ndsi_snow = 0;
         }

//       Check for ice clouds mis-identified as snow. Note modified BTD thresholds for Greenland.
         diff = -99.0;
         diff2 = -99.0;
         sth85_11 = -99.0;
         sth37_11 = -99.0;
         if(rintf(m29) != I_bad && rintf(m31) != I_bad) {
           diff = m29 - m31;
           sth85_11 = sm85_11[0];
           if(rintf(m20) != I_bad && !(pxin.Greenland && pxin.hi_elev)) {
             diff2 = m20 - m31;
             sth37_11 = sm37_11[0];
             if(diff >= sth85_11 && diff2 >= sth37_11) {
               pxin.ndsi_snow = 0;
             }
           }
           else {
             if(pxin.Greenland && pxin.hi_elev) {
               sth85_11 = sm85_11hel[0];
             }
             else {
               sth85_11 = sm85_11[0];
             }
             if(diff >= sth85_11) {
               pxin.ndsi_snow = 0;
             }
           }
         }
//       printf("ice chk: %d %d %f %f\n\n", pxin.Greenland, pxin.hi_elev, diff, sth85_11);

//       Check for water clouds mis-identified as snow.
         diff = -99.0;
         sth37_11 = -99.0;
         if(m20 != I_bad && m31 != I_bad) {
           if( !(pxin.hi_elev && pxin.Greenland) ) {
             diff = m20 - m31;
             if(pxin.hi_elev) {
               sth37_11 = sm37_11hel[0];
             }
             else {
               sth37_11 = sm37_11[0];
             }
             if(diff >= sth37_11) {
               pxin.ndsi_snow = 0;
             }
           }
         }

//       Check value of NIR reflectance.
         if( !pxin.hi_elev && !hi_elev_snow) {
           if(nirband > sm_mnir[0]) {
             pxin.ndsi_snow = 0;
           }

//         Check 11 um vs. surface temperature.
           sfcdif = pxin.sfctmp - m31;
           if(lsf == 1 || lsf == 2) {
//           Land or coast.
             if(sfcdif > sm_lsfcdif[0]) pxin.ndsi_snow = 0;
           }
           else if(lsf == 0 || lsf > 2) {
//           Water.
             if(sfcdif > sm_wsfcdif[0] && pxin.sfctmp > sm_bt1[0]) pxin.ndsi_snow = 0;
           }

         }


       }
     }
   }


}
